package cn.jlsxwkj.common.reader;

import cn.jlsxwkj.common.utils.SplitDocument;
import lombok.Data;
import org.apache.tika.metadata.Metadata;
import org.apache.tika.parser.AutoDetectParser;
import org.apache.tika.parser.ParseContext;
import org.apache.tika.sax.BodyContentHandler;
import org.springframework.ai.document.Document;
import org.springframework.ai.document.DocumentReader;
import org.springframework.ai.reader.ExtractedTextFormatter;
import org.xml.sax.ContentHandler;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.List;

/**
 * @author zh
 * 文档读取
 */
@Data
public class ParagraphDocReader implements DocumentReader {

    private final AutoDetectParser parser;
    private final ContentHandler handler;
    private final Metadata metadata;
    private final ParseContext context;
    private final InputStream inputStream;
    private final String fileName;
    private int windowSize;
    private ExtractedTextFormatter textFormatter;


    public ParagraphDocReader(byte[] bytes,
                              String fileName,
                              int windowSize) {
        this.textFormatter = ExtractedTextFormatter.defaults();
        this.parser = new AutoDetectParser();
        this.handler = new BodyContentHandler(-1);
        this.metadata = new Metadata();
        this.context = new ParseContext();
        this.inputStream = new ByteArrayInputStream(bytes);
        this.windowSize = windowSize;
        this.fileName = fileName;
    }

    @Override
    public List<Document> get() {
        try {
            this.parser.parse(this.inputStream, this.handler, this.metadata, this.context);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return SplitDocument.splitStringToListDocument(this.handler.toString(), this.windowSize, this.fileName);
    }

}
