package cn.jlsxwkj.common.reader;

import cn.hutool.http.HttpRequest;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.jlsxwkj.common.utils.SplitDocument;
import lombok.Data;
import org.springframework.ai.document.Document;
import org.springframework.ai.document.DocumentReader;

import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * @author zh
 */
@Data
public class ParagraphOcrReader implements DocumentReader {

    private final String cnocrUrl;
    private final byte[] bytes;
    private final Charset charset = StandardCharsets.UTF_8;
    private final int windowSize;
    private final String fileName;

    public ParagraphOcrReader(byte[] bytes,
                              String fileName,
                              String cnocrUrl,
                              int windowSize) {
        this.bytes = bytes;
        this.windowSize = windowSize;
        this.cnocrUrl = cnocrUrl;
        this.fileName = fileName;
    }

    @Override
    public List<Document> get() {
        String image = HttpRequest.post(cnocrUrl)
                .form("image", bytes, fileName)
                .execute()
                .body();
        StringBuilder sb = new StringBuilder();
        new JSONArray(new JSONObject(image).get("results"))
                .forEach(x -> sb.append(new JSONObject(x).get("text")).append(" "));
        String document = sb.toString();
        return SplitDocument.splitStringToListDocument(document, this.windowSize, this.fileName);
    }
}