package cn.jlsxwkj.common.utils;

import cn.hutool.core.util.ArrayUtil;
import org.springframework.ai.document.Document;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author jay
 */
public class MergeDocuments {

    /**
     * 合并文档列表
     *
     * @param documentList 文档列表
     * @return 合并后的文档列表
     */
    public static List<Document> mergeDocuments(List<Document> documentList) {
        ArrayList<Document> mergeDocuments = new ArrayList<>();
        //根据文档来源进行分组
        for (var docListEntry : documentList.stream()
                .collect(Collectors.groupingBy(
                        document -> (String) document.getMetadata().get(SplitDocument.FILE_NAME)
                )).entrySet()) {
            //获取最大的段落结束编码
            Optional<Document> documents = docListEntry.getValue().stream()
                    .max(Comparator.comparing(
                            document -> (int) document.getMetadata().get(SplitDocument.END_PARAGRAPH_NUMBER)
                    ));
            if (documents.isEmpty()) {
                continue;
            }
            int maxParagraphNum = (int) documents.get().getMetadata().get(SplitDocument.END_PARAGRAPH_NUMBER);
            //根据最大段落结束编码构建一个用于合并段落的空数组
            String[] paragraphs = new String[maxParagraphNum];
            //用于获取最小段落开始编码
            int minParagraphNum = maxParagraphNum;
            for (Document document : docListEntry.getValue()) {
                //文档内容根据回车进行分段
                String[] tempPs = document.getContent().split("\n");
                //获取文档开始段落编码
                int startParagraphNumber = (int) document.getMetadata().get(SplitDocument.START_PARAGRAPH_NUMBER);
                if (minParagraphNum > startParagraphNumber) {
                    minParagraphNum = startParagraphNumber;
                }
                //将文档段落列表拷贝到合并段落数组中
                System.arraycopy(tempPs, 0, paragraphs, startParagraphNumber - 1, tempPs.length);
            }
            //合并段落去除空值,并组成文档内容
            Document mergeDoc = new Document(ArrayUtil.join(ArrayUtil.removeNull(paragraphs), "\n"));
            //合并元数据
            mergeDoc.getMetadata().putAll(docListEntry.getValue().get(0).getMetadata());
            //设置元数据:开始段落编码
            mergeDoc.getMetadata().put(SplitDocument.START_PARAGRAPH_NUMBER, minParagraphNum);
            //设置元数据:结束段落编码
            mergeDoc.getMetadata().put(SplitDocument.END_PARAGRAPH_NUMBER, maxParagraphNum);
            mergeDocuments.add(mergeDoc);
        }
        return mergeDocuments;
    }
}
