package cn.jlsxwkj.moudles.chat;

import cn.dev33.satoken.annotation.SaIgnore;
import cn.jlsxwkj.common.exception.CustomException;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import reactor.core.publisher.Flux;

/**
 * @author zh
 */
@RestController
@RequestMapping("/ai/rag/")
@Tag(name = "RAG chat")
public class ChatController {

    @Resource
    private ChatService chatService;

    @Operation(summary = "上传文档")
    @PostMapping("/upload")
    @SaIgnore
    public String uploadDoc(@RequestBody MultipartFile file) throws CustomException {
        return chatService.uploadDocument(file);
    }

    @Operation(summary = "搜索文档")
    @PostMapping("/search")
    public String searchDoc(@RequestParam String keyword) {
        return chatService.search(keyword);
    }

    @Operation(summary = "问答文档流")
    @PostMapping(value = "/chatStream", produces = {MediaType.TEXT_EVENT_STREAM_VALUE})
    public Flux<String> chatStream(@RequestParam String message) {
        return chatService.chatStream(message);
    }

    @Operation(summary = "问答文档总结流")
    @PostMapping(value = "/chatSummaryStream", produces = {MediaType.TEXT_EVENT_STREAM_VALUE})
    public Flux<String> chatSummaryStream(@RequestParam(defaultValue = "帮我总结一下上面内容") String message,
                                      @RequestBody MultipartFile file) {
        return chatService.chatSummaryStream(message, file);
    }

    @Operation(summary = "停止问答")
    @GetMapping("/stopChat")
    public void stopChat() {
        chatService.stop();
    }
}
